import glibtop
from utils.Struct import Struct
import libdesklets.convert

"""
This module provides system informations in a user friendly way.
"""

def cpu():

    """
    Provides CPU time information.

    All CPU units are measured in 'jiffies' which are normally 1/100th
    of a second (in which case C{frequency} equals 100), but can also be in
    any other unit. To get seconds, divide them by C{frequency}.

    @rtype:  Struct
    @return: Struct having all the following keys

      - C{total : long}

      Number of clock ticks since system boot.

      - C{user : long}

      Number of clock ticks the system spent in user mode.

      - C{nice : long}

      Number of clock ticks the system spent in user mode (nice).

      - C{sys : long}

      Number of clock ticks the system spent in system mode.

      - C{idle : long}

      Number of clock ticks the system spent in the idle task.

      - C{frequency : long}

      Tick frequency (default is 100).
    """

    buf = glibtop.get_cpu()

    smp = tuple(
        [ Struct(
        total     = buf[glibtop.XCPU_TOTAL][i],
        user      = buf[glibtop.XCPU_USER][i],
        nice      = buf[glibtop.XCPU_NICE][i],
        sys       = buf[glibtop.XCPU_SYS][i],
        idle      = buf[glibtop.XCPU_SYS][i])
          for i in range(0, glibtop.NCPU) if buf[glibtop.XCPU_TOTAL][i] != 0
          ]
        )

    return Struct(
        total     = buf[glibtop.CPU_TOTAL],
        user      = buf[glibtop.CPU_USER],
        nice      = buf[glibtop.CPU_NICE],
        sys       = buf[glibtop.CPU_SYS],
        idle      = buf[glibtop.CPU_SYS],
        frequency = buf[glibtop.CPU_FREQUENCY],
        cpus      = smp
        )



def fsusage(mount_dir):

    """
    'total'
     Total bytes in the filesystem.

     'free'
     Free bytes available to the superuser.

     'avail'
     Free bytes available to ordinary users.

     'files'
     Total file nodes.

     'ffree'
     Free file nodes.

     'block_size'
     Block size in byte.
    """

    buf = glibtop.get_fsusage(mount_dir)

    # just shorter
    bsize = buf[glibtop.FSUSAGE_BLOCK_SIZE]

    # convert bfree and bavail to free and avail bytes
    return Struct(
        total  = buf[glibtop.FSUSAGE_BLOCKS] * bsize,
        free   = buf[glibtop.FSUSAGE_BFREE]  * bsize,
        avail  = buf[glibtop.FSUSAGE_BAVAIL] * bsize,
        files  = buf[glibtop.FSUSAGE_FILES],
        ffree  = buf[glibtop.FSUSAGE_FFREE],
        block_size = bsize
        )



def loadavg():

    """
    Provides load average informations.

    @rtype:  Struct
    @return: Struct having all the following keys

      - C{loadavg5  : float}
      - C{loadavg10 : float}
      - C{loadavg15 : float}

      Number of jobs running simultaneously averaged over 1, 5 and 15
      minutes.

      - C{nr_running : long}

      Number of tasks currently running.

      - C{nr_tasks : long}

      Total number of tasks.

      - C{last_pid : long}

      Last PID.
    """

    buf = glibtop.get_loadavg()

    return Struct(
        loadavg5   = buf[glibtop.LOADAVG_LOADAVG][0],
        loadavg10  = buf[glibtop.LOADAVG_LOADAVG][1],
        loadavg15  = buf[glibtop.LOADAVG_LOADAVG][2],
        nr_running = buf[glibtop.LOADAVG_NR_RUNNING],
        nr_task    = buf[glibtop.LOADAVG_NR_TASKS],
        last_pid   = buf[glibtop.LOADAVG_LAST_PID]
        )



def mem():

    """
    Provides memory information.

    All memory units are in bytes.

    @rtype:  Struct
    @return: Struct having all the following keys

      - C{total : long}

      Total physical memory.

      - C{used : long}

      Used memory size.

      - C{free : long}

      Free memory size.

      - C{shared : long}

      Shared memory size.

      - C{buffer : long}

      Size of buffers.

      - C{cached : long}

      Size of cached memory.

      - C{user : long}

      Memory used from user processes.
      This is normally M{total - free - shared - buffer - cached}.

      - C{locked : long}

      Memory in locked segments.
      """

    buf = glibtop.get_mem()

    return Struct(
        total  = buf[glibtop.MEM_TOTAL],
        used   = buf[glibtop.MEM_USED],
        free   = buf[glibtop.MEM_FREE],
        shared = buf[glibtop.MEM_SHARED],
        buffer = buf[glibtop.MEM_BUFFER],
        cached = buf[glibtop.MEM_CACHED],
        user   = buf[glibtop.MEM_USER],
        locked = buf[glibtop.MEM_LOCKED]
        )



def mountlist(all_fs = False):

    """
    Returns the list of (all) mount points.

    @param all_fs:  specifies whether information about all
    filesystems should be returned; this will include filesystem types like
    'autofs' and 'procfs'. You should not use this in disk usage programs,
    but it can be useful to get a list of all currently mounted filesystems.

    @type all_fs: bool

    @rtype:  list
    @return: list of Struct having all the following keys

      - C{devname : str}

      Full pathname (such as '/dev/sdb1' for instance) to the mounted
      device.

      - C{mountdir : str}

      Full pathname of the mountpoint (such as '/usr/local' for
      instance).

      - C{type : str}

      Filesystem type as a textual string (such as 'ext2fs').
    """

    def mountentry(e):

        # e[0] -> dev seems to be meaningless
        # and is not documented
        # no CONSTANT in GLIBTOP
        # TODO: submit a patch for libgtop

        return Struct(
            devname  = e[1],
            mountdir = e[2],
            type     = e[3]
        )

    return [ mountentry(e) for e in glibtop.get_mountlist(all_fs) ]



def net_load(interface):

    """
    Returns network statistics for interface 'interface' (which is the
    same than in 'ifconfig')

    @param interface: specifies the network interface.
    @type  interface: str

      - C{if_flags : long}

      Interface flags.

      GLIBTOP_IF_FLAGS_UP,
      GLIBTOP_IF_FLAGS_BROADCAST,
      GLIBTOP_IF_FLAGS_DEBUG,
      GLIBTOP_IF_FLAGS_LOOPBACK,
      GLIBTOP_IF_FLAGS_POINTOPOINT,
      GLIBTOP_IF_FLAGS_RUNNING,
      GLIBTOP_IF_FLAGS_NOARP,
      GLIBTOP_IF_FLAGS_PROMISC,
      GLIBTOP_IF_FLAGS_ALLMULTI,
      GLIBTOP_IF_FLAGS_OACTIVE,
      GLIBTOP_IF_FLAGS_SIMPLEX,
      GLIBTOP_IF_FLAGS_LINK0,
      GLIBTOP_IF_FLAGS_LINK1,
      GLIBTOP_IF_FLAGS_LINK2,
      GLIBTOP_IF_FLAGS_ALTPHYS,
      GLIBTOP_IF_FLAGS_MULTICAST

      - C{mtu : long}

      Maximum Transfer Unit (MTU)

      - C{subnet : long}

      Subnet Address

      - C{address : long}

      Interface Address

      - C{packets_in : long}

      Total number of incoming packets

      - C{packets_out : long}

      Total number of outgoing packets

      - C{packets_total : long}

      Total number of packets

      - C{bytes_in : long}

      Total number of incoming bytes

      - C{bytes_out : long}

      Total number of outgoing bytes

      - C{bytes_total : long}

      Total number of bytes

      - C{errors_in : long}

      Total number of errors in incoming direction

      - C{errors_out : long}

      Total number of errors in outgoing direction

      - C{errors_total : long}

      Total number of errors

      - C{collisions : long}

      Total number of collisions
    """

    buf = glibtop.get_netload(interface)

    conv = libdesklets.convert.ipv4_to_dotted_quad

    return Struct(if_flags      = buf[glibtop.NETLOAD_IF_FLAGS],
                  mtu           = buf[glibtop.NETLOAD_MTU],
                  subnet        = conv(buf[glibtop.NETLOAD_SUBNET]),
                  address       = conv(buf[glibtop.NETLOAD_ADDRESS]),
                  packets_in    = buf[glibtop.NETLOAD_PACKETS_IN],
                  packets_out   = buf[glibtop.NETLOAD_PACKETS_OUT],
                  packets_total = buf[glibtop.NETLOAD_PACKETS_TOTAL],
                  bytes_in      = buf[glibtop.NETLOAD_BYTES_IN],
                  bytes_out     = buf[glibtop.NETLOAD_BYTES_OUT],
                  bytes_total   = buf[glibtop.NETLOAD_BYTES_TOTAL],
                  errors_in     = buf[glibtop.NETLOAD_ERRORS_IN],
                  errors_out    = buf[glibtop.NETLOAD_ERRORS_OUT],
                  errors_total  = buf[glibtop.NETLOAD_ERRORS_TOTAL],
                  collisions    = buf[glibtop.NETLOAD_COLLISIONS]
                  )



def ppp(device):

    """
    Provides Point to Point Protocol (PPP) informations.

    @param devive: specifies the device. Should be positive.
    @type  device: int

    @rtype: Struct
    @return: list of Struct having all the following keys

      - C{state : long}

      PPP_STATE_UNKNOWN,
      PPP_STATE_HANGUP,
      PPP_STATE_ONLINE

      - C{bytes_in : long}

      Number of input bytes

      - C{bytes_out : long}
      Number of output bytes
    """

    buf = glibtop.get_ppp(device)

    return Struct(
        state     = buf[glibtop.PPP_STATE],
        bytes_in  = buf[glibtop.PPP_BYTES_IN],
        bytes_out = buf[glibtop.PPP_BYTES_OUT]
        )



def proc_args(pid):

    """
    @param pid: PID
    @type  pid: long

    @rtype : tuple of str
    @return : tuple with all command line arguments of process pid
    """

    return glibtop.get_proc_args(pid)



def proc_kernel(pid):

    """
    """

    buf = glibtop.get_proc_kernel(pid)

    return Struct(
        k_flags  = buf[glibtop.PROC_KERNEL_K_FLAGS],
        min_flt  = buf[glibtop.PROC_KERNEL_MIN_FLT],
        maj_flt  = buf[glibtop.PROC_KERNEL_MAJ_FLT],
        cmin_flt = buf[glibtop.PROC_KERNEL_CMIN_FLT],
        cmaj_flt = buf[glibtop.PROC_KERNEL_CMAJ_FLT],
        kstk_esp = buf[glibtop.PROC_KERNEL_KSTK_ESP],
        kstk_eip = buf[glibtop.PROC_KERNEL_KSTK_EIP],
        nwchain  = buf[glibtop.PROC_KERNEL_NWCHAIN],
        wchain   = buf[glibtop.PROC_KERNEL_WCHAIN]
        )



def proc_list(which, arg):

    """
    KERN_*
    """

    return glibtop.get_proclist(which, arg)



def proc_map(pid):

    """
    """

    def mapentry(e):

        return Struct(
            start    = e[glibtop.MAP_ENTRY_START],
            end      = e[glibtop.MAP_ENTRY_END],
            offset   = e[glibtop.MAP_ENTRY_OFFSET],
            perm     = e[glibtop.MAP_ENTRY_PERM],
            inode    = e[glibtop.MAP_ENTRY_INODE],
            device   = e[glibtop.MAP_ENTRY_DEVICE],
            filename = e[glibtop.MAP_ENTRY_FILENAME],
            )

    return [ mapentry(e) for e in glibtop.get_proc_args(pid) ]



def proc_mem(pid):

    """
    """

    buf = glibtop.get_proc_mem(pid)

    return Struct(
        size     = buf[glibtop.PROC_MEM_SIZE],
        vsize    = buf[glibtop.PROC_MEM_VSIZE],
        resident = buf[glibtop.PROC_MEM_RESIDENT],
        share    = buf[glibtop.PROC_MEM_SHARE],
        rss      = buf[glibtop.PROC_MEM_RSS],
        rss_rlim = buf[glibtop.PROC_MEM_RSS_RLIM],
        )



def proc_segment(pid):

    """
    """
    buf = glibtop.get_proc_segment(pid)

    return Struct(
        text_rss    = buf[glibtop.PROC_SEGMENT_TEXT_RSS],
        shlib_rss   = buf[glibtop.PROC_SEGMENT_SHLIB_RSS],
        data_rss    = buf[glibtop.PROC_SEGMENT_DATA_RSS],
        stack_rss   = buf[glibtop.PROC_SEGMENT_STACK_RSS],
        dirty_size  = buf[glibtop.PROC_SEGMENT_DIRTY_SIZE],
        start_code  = buf[glibtop.PROC_SEGMENT_START_CODE],
        end_code    = buf[glibtop.PROC_SEGMENT_END_CODE],
        start_stack = buf[glibtop.PROC_SEGMENT_START_STACK]
        )



def proc_signal(pid):

    """
    """

    buf = glibtop.get_proc_signal(pid)

    return Struct()



def proc_state(pid):

    """
    """

    buf = glibtop.get_proc_state(pid)

    return Struct(
        cmd   = buf[glibtop.PROC_STATE_CMD],
        state = buf[glibtop.PROC_STATE_STATE],
        uid   = buf[glibtop.PROC_STATE_UID],
        gid   = buf[glibtop.PROC_STATE_GID]
        )



def proc_time(pid):

    """
    """

    buf = glibtop.get_proc_time(pid)

    return Struct(
        start_time    = buf[glibtop.PROC_TIME_START_TIME],
        rtime         = buf[glibtop.PROC_TIME_RTIME],
        utime         = buf[glibtop.PROC_TIME_UTIME],
        stime         = buf[glibtop.PROC_TIME_STIME],
        cutime        = buf[glibtop.PROC_TIME_CUTIME],
        cstime        = buf[glibtop.PROC_TIME_CSTIME],
        timeout       = buf[glibtop.PROC_TIME_TIMEOUT],
        it_real_value = buf[glibtop.PROC_TIME_IT_REAL_VALUE]
        )



def proc_uid(pid):

    """
    """

    buf = glibtop.get_proc_uid(pid)

    return Struct(
          uid      = buf[glibtop.PROC_UID_UID],
          euid     = buf[glibtop.PROC_UID_EUID],
          gid      = buf[glibtop.PROC_UID_GID],
          egid     = buf[glibtop.PROC_UID_EGID],
          pid      = buf[glibtop.PROC_UID_PID],
          ppid     = buf[glibtop.PROC_UID_PPID],
          pgrp     = buf[glibtop.PROC_UID_PGRP],
          session  = buf[glibtop.PROC_UID_SESSION],
          tty      = buf[glibtop.PROC_UID_TTY],
          tpgid    = buf[glibtop.PROC_UID_TPGID],
          priority = buf[glibtop.PROC_UID_PRIORITY],
          nice     = buf[glibtop.PROC_UID_NICE]
          )



def swap():

    """
    Provides swap information.

    All memory units are in bytes.

    @rtype:  Struct
    @return: Struct having all the following keys

      - C{total : long}

      Total swap space in the system.

      - C{used : long}

      Used swap space.

      - C{free : long}

      Free swap space.

      You can use 'pagein' and 'pageout' to get some measure about how
      much the system is swapping at the moment. They're increased each time
      a page is swapped in or out, so you need to save this values, wait a
      little bit, get them again and then compare the two results to find out
      how much the system swapped in the meantime.

      - C{pagein : long}

      Total number of swap pages that have been brought in since system
      boot

      - C{pageout : long}

      Total number of swap pages that have been brought out since system
      boot
    """

    buf = glibtop.get_swap()

    return Struct(
        total   = buf[glibtop.SWAP_TOTAL],
        used    = buf[glibtop.SWAP_USED],
        free    = buf[glibtop.SWAP_FREE],
        pagein  = buf[glibtop.SWAP_PAGEIN],
        pageout = buf[glibtop.SWAP_PAGEOUT]
        )



def uptime():

    """
    Provides uptime informations.

    @rtype:  Struct
    @return: Struct having all the following keys

      - C{uptime : float}

      Time in seconds since system boot.

      - C{idletime : float}

      Time in seconds the system spent in the idle task since system
      boot.
    """

    buf = glibtop.get_uptime()

    return Struct(
        uptime    = buf[glibtop.UPTIME_UPTIME],
        idletime  = buf[glibtop.UPTIME_IDLETIME],
        boot_time = buf[glibtop.UPTIME_BOOT_TIME]
        )
