from main import AUTHORS, COPYRIGHT, DESCRIPTION, DOCUMENTERS, HOME, ICON, NAME
from main import POSITIONSFILE, PURGE_KEY, USERHOME, VERSION, LOGFILE
from AboutDialog import AboutDialog
from config import settings
from config.ConfigManager import ConfigManager
from display.Display import Display
from display.Window import Window
from display.Plug import Plug
from factory.DisplayFactory import DisplayFactory
from TrayIcon import TrayIcon
from utils import dialog
from utils import singleton
from utils import vfs

import RemoteSocket

import gtk
import shutil
import signal
import os
import sys


# This class starts, restarts and kills Displays
class Starter:

    __DISPLAY_TYPE_WINDOW = "window"
    __DISPLAY_TYPE_PLUG = "plug"


    def __init__(self):

        # the About dialog
        self.__about_dialog = AboutDialog(os.path.join(HOME, "data"))

        # the command which gets called when the user removes a display
        self.__remove_command = ""

        # the command which gets called when the session restarts to restart
        # gDesklets
        self.__startup_command = ""

        # the set of open displays as a hashtable "id -> display"
        self.__open_displays = {}
        # the paths of the display files "id -> path"
        self.__display_paths = {}
        # the saved positions of the displays "id -> (x, y)"
        self.__positions = {}

        self.__factory = singleton.get(DisplayFactory)
        self.__config  = singleton.get(ConfigManager)

        # the communication socket
        self.__socket = RemoteSocket.RemoteSocket()

        # set the message handlers for the socket
        self.__socket.add_message_handler(RemoteSocket.COMMAND_OPEN_DISPLAY,
                                          self.__handle_open_display)
        self.__socket.add_message_handler(
            RemoteSocket.COMMAND_OPEN_DISPLAY_WITH_ID,
            self.__handle_open_display_with_id)
        self.__socket.add_message_handler(RemoteSocket.COMMAND_OPEN_PLUG,
                                          self.__handle_open_plug)
        self.__socket.add_message_handler(RemoteSocket.COMMAND_CLOSE_DISPLAY,
                                          self.__handle_close_display)
        self.__socket.add_message_handler(RemoteSocket.COMMAND_VERSION,
                                          self.__handle_version)
        self.__socket.add_message_handler(RemoteSocket.COMMAND_SHUTDOWN,
                                          self.__handle_shutdown)
        self.__socket.add_message_handler(RemoteSocket.COMMAND_DISPLAYS,
                                          self.__handle_displays)
        self.__socket.add_message_handler(RemoteSocket.COMMAND_DISPLAY_LIST,
                                          self.__handle_display_list)
        self.__socket.add_message_handler(RemoteSocket.COMMAND_GEOMETRY,
                                          self.__handle_geometry)
        self.__socket.add_message_handler(
            RemoteSocket.COMMAND_SET_REMOVE_COMMAND,
            self.__handle_set_remove_command)
        self.__socket.add_message_handler(
            RemoteSocket.COMMAND_SET_STARTUP_COMMAND,
            self.__handle_set_startup_command)


        self.__load_positions()

        # socket ready, start handling requests
        self.__socket.start()

        # setup a nice systray icon
        trayicon = TrayIcon()
        trayicon.set_menu([(gtk.STOCK_PROPERTIES, _("_Manage desklets"),
                            self.__handle_manage),
                           (None, _("_View log"), self.__handle_show_log),
                           (None, _("_About..."), self.__handle_about),
                           (),
                           (gtk.STOCK_CLOSE, _("_Stop daemon"),
                            self.__handle_shutdown)])

        # save positions from time to time
        def f():
            self.__save_positions()
            return True

        gtk.timeout_add(3000, f)



    #
    # Reacts on observer messages from the display.
    #
    def __on_display_action(self, src, cmd, *args):

        if (cmd == src.OBS_CLOSE):
            ident = args[0]
            self.__close_display(ident)
            del src

        elif (cmd == src.OBS_RESTART):
            ident = args[0]
            x, y, w, h = src.get_geometry()
            self.__positions[ident] = (int(x), int(y))
            path = self.__display_paths[ident]
            self.__remove_display(ident)
            gtk.timeout_add(250, self.__add_display, ident, path,
                            self.__DISPLAY_TYPE_WINDOW)
            del src



    #
    # Adds the given display.
    #
    def __add_display(self, ident, path, displaytype):

        dsp = None
        container = None

        if (ident not in self.__open_displays):
            dsp = self.__create_display(ident, path)

            if (dsp):
                self.__open_displays[ident] = dsp
                self.__display_paths[ident] = path

                print "Info: Adding [%s] with ID '%s' to the list of " \
                      "displays." % (path, ident)
                dsp.add_observer(self.__on_display_action)

                if (displaytype == self.__DISPLAY_TYPE_WINDOW):
                    container = Window()
                    try:
                        x, y = self.__positions[ident]
                        dsp.set_position(x, y)
                    except KeyError:
                        dsp.set_position()

                    dsp.set_container(container)

                elif (displaytype == self.__DISPLAY_TYPE_PLUG):
                    container = Plug()
                    dsp.set_container(container)
                    container.show()

        return container



    #
    # Creates and returns a new display from the given data, or None in case
    # of an error.
    #
    def __create_display(self, ident, path):

        try:
            if (not vfs.exists(path)): raise IOError(_("File does not exist."))
            fd = vfs.open(path, "r")
            data = vfs.read_all(fd)
            fd.close()

        except:
            import traceback; traceback.print_exc()
            print >>sys.stderr, "Could not open display file [%s]." % (path,)
            dialog.warning(_("Could not open display file '%(path)s'") % vars(),
                           _("The display file could not be opened because "
                             "the file was not readable."), False)
            self.__close_display(ident)
            return

        rep = os.path.dirname(path)

        vfs.chdir(rep)
        display = self.__factory.create_display(ident, data)

        if (not display):
            dialog.warning(_("Invalid display file '%(path)s'") % vars(),
                           _("The display file contains invalid data and "
                             "could not be loaded."), False)
            self.__close_display(ident)

        return display



    #
    # Removes the given display.
    #
    def __remove_display(self, ident):

        if (ident in self.__open_displays):
            # better use python2.3 dict.pop
            d = self.__open_displays[ident]
            x, y, w, h = d.get_geometry()
            self.__positions[ident] = (int(x), int(y))
            if d:
                print "Info: Removing [%s] with ID '%s' from the list of " \
                      "displays." % (d.get_path(), ident)
                d.remove_display()
            del self.__open_displays[ident]
            del self.__display_paths[ident]



    #
    # Closes the given display.
    #
    def __close_display(self, ident):

        self.__remove_display(ident)
        self.__purge_config(ident)
        if (self.__remove_command):
            os.system("%s %s &" % (self.__remove_command, ident))

        try:
            del self.__positions[ident]
        except:
            pass



    #
    # Purges the given config entry.
    #
    def __purge_config(self, ident):

        # get paths to purge
        # empty are ignored
        purge = filter(None, self.__config.get(ident, PURGE_KEY).split(","))
        for p in purge:
            # relative paths are relative to USERHOME
            try:
                p = os.path.normpath(os.path.join(USERHOME, p))
            except OSError, exc:
                import traceback; traceback.print_exc()
                continue
            # make sure that only paths below USERHOME can be purged
            if (p.startswith(USERHOME) and os.path.exists(p)):
                print "Info: Purging path [%s]." % (p,)
                # FIXME
                #shutil.rmtree(p)
                # hey this codes does nothing !

        print "Info: Purging ID '%s'." % (ident,)
        self.__config.clear(ident)



    #
    # Loads the display positions from file.
    #
    def __load_positions(self):

        try:
            lines = open(POSITIONSFILE, "r").readlines()
        except:
            # no positions
            return

        for l in lines:
            if (not l.strip()): continue
            ident, x, y = l.split()
            self.__positions[ident] = (int(x), int(y))



    #
    # Saves the current display positions to a file.
    #
    def __save_positions(self):

        changed = False

        for ident, display in self.__open_displays.items():
            x, y, w, h = display.get_geometry()
            if (self.__positions.get(ident) != (int(x), int(y))):
                changed = True
            self.__positions[ident] = (int(x), int(y))

        if (not changed): return

        data = ""
        for ident, pos in self.__positions.items():
            x, y = pos
            data += "%s %d %d\n" % (ident, x, y)
        try:
            open(POSITIONSFILE, "w").write(data)
        except IOException, exc:
            import traceback; traceback.print_exc()
            print >>sys.stderr, "Could not write the position file [%s]." \
                                                           (POSITIONSFILE,)



    def __handle_open_display(self, path):

        ident = Display.make_id()
        self.__handle_open_display_with_id(path, ident)

        return (ident,)



    def __handle_open_display_with_id(self, path, ident):

        self.__add_display(ident, path, self.__DISPLAY_TYPE_WINDOW)

        return (None,)



    def __handle_open_plug(self, path):

        import time
        ident = str(time.time())
        plug = self.__add_display(ident, path, self.__DISPLAY_TYPE_PLUG)
        xid = plug.get_xembed_id()

        return (ident, xid)



    def __handle_close_display(self, ident):

        self.__remove_display(ident)

        return (None,)



    def __handle_version(self):

        return (NAME, VERSION)



    def __handle_shutdown(self, *args):

        self.__save_positions()

        for ident, display in self.__open_displays.items():
            try:
                display.remove_display()
            except:
                pass
        gtk.main_quit()
        return (None,)



    def __handle_manage(self, *args):

        cmd = os.path.join(HOME, "gdesklets-shell")
        os.system(cmd + " &")
        


    def __handle_about(self, *args):

        self.__about_dialog.show()
        


    def __handle_show_log(self, *args):

        from LogViewer import LogViewer
        viewer = LogViewer(LOGFILE)


    def __handle_displays(self):

        return self.__open_displays.keys()



    def __handle_display_list(self):

        return self.__open_display.values()



    def __handle_geometry(self, ident):

        display = self.__open_displays[ident]

        return display.get_geometry()



    def __handle_set_remove_command(self, command):

        # only allow setting the remove command once for security reasons
        if (not self.__remove_command):
            self.__remove_command = command

        return (None,)



    def __handle_set_startup_command(self, command):

        # only allow setting the startup command once for security reasons
        if (not self.__startup_command):
            self.__startup_command = command

            # connect to gnome-session
            try:
                import gnome.ui
                # does the required method exist?
                gnome.ui.Client.set_restart_command
                client = gnome.ui.master_client()
                if (client):
                    client.set_restart_style(gnome.ui.RESTART_IF_RUNNING)
                    cmd = command.split()
                    try:
                        client.set_restart_command(*cmd)
                    except:
                        client.set_restart_command(len(cmd), cmd)
            except:
                import traceback; traceback.print_exc()
                print >>sys.stderr, "Could not connect to gnome-session."
                print _("Could not connect to gnome-session. "
                        "GNOME session management "
                        "will not be available for this application.")

        return (None,)
