#
# Client module for the gDesklets daemon. Use this module to talk to the
# daemon from external programs.
#

from main import COPYRIGHT, HOME, NAME, SOCKET_PATH, USERHOME, VERSION
from remotecommands import *
import utils.xdr as xdr

import os
import sys
import socket
import time

from utils import i18n
import __builtin__
__builtin__._ = i18n.Translator("gdesklets")


class RemoteConnection:


    def __init__(self, path):

        self.__sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.__sock.connect(path)



    #
    # Sends the given command to the given socket and returns the answer.
    #
    def __send(self, command, *args):

        xdr.send(self.__sock, command, *args)

        return (xdr.recv(self.__sock))



    def open_display(self, path):

        """
        @param path: The (VFS-)path of the display file to open.
        @type  path: str

        @return: The ID of the new display.
        @rtype : str
        """

        return self.__send(COMMAND_OPEN_DISPLAY, path)[0]



    def open_display_with_id(self, path, ident):

        """
        @param path: The (VFS-)path of the display file to open.
        @type  path: str
        @param ident: The ID of the display.
        @type  ident: str
        """

        return self.__send(COMMAND_OPEN_DISPLAY_WITH_ID, path, ident)[0]



    def open_plug(self, path):

        """
        Opens a display file as an embeddable plug. Returns the ID of the plug,
        compliant with the XEmbed protocol.

        @param path: The (VFS-)path of the display file to open.
        @type  path: str

        @return: (id, xid) The ID of the new display and the XID of the plug.
        @rtype : str
        """

        ident, xid = self.__send(COMMAND_OPEN_PLUG, path)

        return (ident, long(xid))



    def close_display(self, ident):

        """
        @param ident: The ID of the display.
        @type  ident: str
        """

        self.__send(COMMAND_CLOSE_DISPLAY, ident)



    def version(self):

        """
        @return:  Tuple with the name and version of the gDesklets daemon
        @rtype : (str, str)
        """

        return self.__send(COMMAND_VERSION)



    def shutdown(self):

        """
        """

        self.__send(COMMAND_SHUTDOWN)



    def displays(self):

        """
        @return: The list of the IDs of the currently open displays.
        @rtype : tuple
        """

        return self.__send(COMMAND_DISPLAYS)



    def display_list(self):

        """
        @return: The list of currently open displays.
        @rtype: list
        """

        return (self.__send(COMMAND_DISPLAY_LIST))



    def geometry(self, ident):

        """
        @param ident: The ID of the display.
        @type  ident: str

        @return: Tuple of the x, y, width, height.
        @rtype : tuple(int, int, int, int)
        """

        x, y, w, h = self.__send(COMMAND_GEOMETRY, ident)

        return (int(x), int(y), int(w), int(h))



    def set_remove_command(self, cmd):

        """
        Sets the remove command which will be run whenever the user removes
        a display. This can only be set once in a daemon's lifetime for
        security reasons.

        @param cmd:  The command.
        @type  path: str
        """

        self.__send(COMMAND_SET_REMOVE_COMMAND, cmd)



    def set_startup_command(self, cmd):

        """
        Sets the startup command which will be run when the session gets
        loaded. This can only be set once in a daemon's lifetime for security
        reasons.

        @param cmd:  The command.
        @type  path: str
        """

        self.__send(COMMAND_SET_STARTUP_COMMAND, cmd)



def print_about():

    """ Prints the about text. """

    print NAME + " " + VERSION
    print COPYRIGHT.replace(u"\xa9", u"(C)")
    print
    print _("This software is licensed under the terms of the GNU GPL.")
    print



def get_daemon(display = None):

    """ Returns the daemon. If there is no daemon running, start a new one. """

    if (not display): display = os.environ["DISPLAY"]

    path = os.path.join(SOCKET_PATH, display)
    daemon = None

    boottime = time.time()


    if(os.path.exists(path)):
        try:
            connection = RemoteConnection(path)
            daemon = connection
        except:
            os.unlink(path)

    if (not os.path.exists(path)):

        daemon = os.path.join(HOME, "gdesklets-daemon")
        if ("--translucent" in sys.argv):
            os.system(daemon + " --translucent" + " &")
        else:
            os.system(daemon + " &")

        pos = 0
        delta = 1
        for i in range(100):
            left = pos
            right = 10 - pos
            bar = " " * left + "###" + " " * right
            sys.stdout.write("\r" + _("Connecting to daemon [%s]") % bar)
            sys.stdout.flush()
            pos += delta
            if (pos == 0 or pos == 10): delta = -delta
            try:
                connection = RemoteConnection(path)
                daemon = connection
                if (daemon): break
            except:
                pass
            time.sleep(0.1)
        #end for
        print "\r"+ " " * 40 + "\r",
        if (not daemon):
	    sys.exit(_("Cannot establish connection to daemon: timeout!"));


    elapsed = int(1000000*(time.time() - boottime))

    if (elapsed < 1000):
        print _("Connected to daemon in %d microseconds.") % (elapsed)
    else:
        elapsed /= 1000
        print _("Connected to daemon in %d milliseconds.") % (elapsed)

    return daemon

