import gtk
import sys
import traceback
from cStringIO import StringIO
import NullObject

#
# Functions for creating every kind of dialog. They are for internal use.
#


# we only want to display one dialog at a time, so let's queue them
_dialog_queue = []

# remember the previous message to avoid displaying the same message twice
# in a sequence
_last_message = None


#
# Displays a HIGified message in the given dialog.
#
def _set_message(dialog, primary, secondary):

    # create a HIG compliant message
    message = "<span weight=\"bold\" size=\"larger\">%(primary)s</span>\n" \
              "\n" \
              "%(secondary)s" % vars()

    # HIG says: no title
    dialog.set_title("")

    hbox = dialog.vbox.get_children()[0]
    lbl = hbox.get_children()[1]
    hbox.remove(lbl)
    align = gtk.Alignment(0.0, 0.0)
    align.show()
    align.add(lbl)

    vbox = gtk.VBox()
    vbox.show()
    hbox.add(vbox)

    lbl.set_markup(message)
    vbox.add(align)



#
# Adds a details button to the given dialog.
#
def _set_details(dialog, details):

    def expand(src, body, icon):
        if (src.get_active()):
            icon.set_from_stock(gtk.STOCK_ZOOM_OUT, gtk.ICON_SIZE_BUTTON)
            body.show()
        else:
            icon.set_from_stock(gtk.STOCK_ZOOM_IN, gtk.ICON_SIZE_BUTTON)
            body.hide()

    vbox = dialog.vbox.get_children()[0].get_children()[1]

    vbox2 = gtk.VBox()
    vbox2.show()
    vbox.pack_end(vbox2, False, False, 12)

    align = gtk.Alignment(0.0, 0.0)
    align.show()
    expander = gtk.ToggleButton()

    hbox = gtk.HBox()
    hbox.show()
    icon = gtk.Image()
    icon.set_from_stock(gtk.STOCK_ZOOM_IN, gtk.ICON_SIZE_BUTTON)
    icon.show()
    hbox.add(icon)
    lbl = gtk.Label(_("Details (%d lines)") % len(details.splitlines()))
    lbl.show()
    hbox.add(lbl)
    expander.add(hbox)

    expander.show()
    align.add(expander)
    vbox2.pack_start(align, False, False)

    viewport = gtk.ScrolledWindow()
    viewport.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
    vbox2.pack_end(viewport)

    lbl = gtk.Label(details)
    lbl.set_selectable(True)
    lbl.show()

    nil, height = lbl.size_request()
    viewport.set_size_request(-1, min(height, 320))
    viewport.add_with_viewport(lbl)

    expander.connect("toggled", expand, viewport, icon)


#
# Queues the given dialog for displaying.
#
def _queue_dialog(dialog):

    def destroy(*args): dialog.destroy()

    def proceed(*args):
        _dialog_queue.pop(0)
        if (_dialog_queue): _dialog_queue[0].show()

    ok_button = dialog.get_children()[0].get_children()[-1].get_children()[-1]
    ok_button.connect("clicked", destroy)
    dialog.connect("destroy", proceed)

    # display the dialog immediately if there are no others in the queue
    if (not _dialog_queue): dialog.show()
    _dialog_queue.append(dialog)



#
# Displays an error dialog. Errors are critical and the program terminates
# afterwards.
#
def error(primary, secondary):

    dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
                               gtk.BUTTONS_OK)
    # HIG says: don't show up in the taskbar
    dialog.set_property("skip-taskbar-hint", 1)

    _set_message(dialog, primary, secondary)
    gtk.threads_enter()
    dialog.run()
    gtk.threads_leave()
    sys.exit(4)


#
# Displays a warning dialog.
#
def warning(primary, secondary, show_traceback = True, force = False):
    global _last_message
    
    # don't show the same dialog twice in a sequence
    if (force): _last_message = ""
    if (_last_message == (primary, secondary)): return
    else: _last_message = (primary, secondary)

    dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING,
                               gtk.BUTTONS_OK)
    dialog.set_property("skip-taskbar-hint", 1)
    if (show_traceback):
        fd = StringIO()
        traceback.print_exc(file = fd)
        details = fd.getvalue()
        fd.close()
    else:
        details = ""

    _set_message(dialog, primary, secondary)
    if (details and not details.startswith("None")):
        _set_details(dialog, details)
    _queue_dialog(dialog)



#
# Displays an information dialog.
#
def info(primary, secondary):

    dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO,
                               gtk.BUTTONS_OK)
    _set_message(dialog, primary, secondary)
    _queue_dialog(dialog)



def fileselector(title, callback_ok, callback_cancel, *args):

    def handler(src, response):
        if (response == gtk.RESPONSE_OK):
            if (callback_ok): callback_ok(src, *args)
        else:
            if (callback_cancel): callback_cancel(src, *args)
            else: src.destroy()


    # do we have FileChooserDialog available?
    try:
        fsel = gtk.FileChooserDialog(title, None, gtk.FILE_CHOOSER_ACTION_OPEN,
                                     (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                      gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        fsel.connect("response", handler)

    # no, then use the old FileSelection
    except:
        def f(btn, fsel, response): handler(fsel, response)

        fsel = gtk.FileSelection()
        
        if (title): fsel.set_title(title)
        fsel.ok_button.connect("clicked", f, fsel, gtk.RESPONSE_OK)
        fsel.cancel_button.connect("clicked", f, fsel, gtk.RESPONSE_CANCEL)

    fsel.show()



def progress(text, parent = None):

    dialog = gtk.Dialog("", parent, 0, ())
    vbox = gtk.VBox()
    vbox.show()
    dialog.vbox.pack_start(vbox, 0, 0, 0)

    lbl = gtk.Label("")
    lbl.set_markup(text)
    lbl.show()
    vbox.add(lbl)

    bar = gtk.ProgressBar()
    bar.show()
    vbox.add(bar)

    dialog.show()

    def f(value):
        value = max(0, value)

        if (value >= 100):
            dialog.destroy()
        else:
            bar.set_fraction(value / 100.0)
            bar.set_text("%i%%" % value)

        while (gtk.events_pending()): gtk.mainiteration()


    return f


#
# Displays a question dialog.
#
def question(icon, primary, secondary, button1, cb1, button2, cb2):

    dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_QUESTION, gtk.BUTTONS_NONE)
    _set_message(dialog, primary, secondary)

    def create_on_clicked(cb):

        def on_clicked(*args):
            cb(*args)
            dialog.destroy()

        return on_clicked

    button1.connect("clicked", create_on_clicked(cb1))
    button2.connect("clicked", create_on_clicked(cb2))

    dialog.action_area.pack_start(button1, True, True, 0)
    dialog.action_area.pack_start(button2, True, True, 0)

    if (icon):
        img = dialog.vbox.get_children()[0].get_children()[0]
        img.set_from_stock(icon, gtk.ICON_SIZE_DIALOG)

    dialog.show_all()



